"""Democritus functions to interact with Hypothesis."""

import ipaddress
from functools import partial

from hypothesis.strategies._internal.ipaddress import ip_addresses

from d8s_hypothesis import hypothesis_get_strategy_results


def is_ip_address(text: str) -> bool:
    """Determine whether or not the given text is an ip address."""
    try:
        ipaddress.ip_address(text)
    except ValueError:
        return False
    else:
        return True


def test_hypothesis_get_strategy_results_dates():
    ipv4_addresses_func = partial(ip_addresses, v=4)
    results = hypothesis_get_strategy_results(ipv4_addresses_func)
    assert len(results) == 10
    for result in results:
        assert is_ip_address(result)

    # try specifying a specific number
    ipv4_addresses_func = partial(ip_addresses, v=4)
    results = hypothesis_get_strategy_results(ipv4_addresses_func, n=12)
    assert len(results) == 12
    for result in results:
        assert is_ip_address(result)



"""Democritus functions for working with hypothesis."""

from hypothesis import given, settings


def hypothesis_get_strategy_results(strategy, *args, n: int = 10, **kwargs):
    """Return the given n of results from the given hypothesis strategy.

    For a list of hypothesis strategies, see: https://hypothesis.readthedocs.io/en/latest/data.html.

    """

    class A:  # pylint: disable=R0903
        def __init__(self):
            self.l = []  # noqa:E741

        @given(strategy(*args))
        @settings(max_examples=n, **kwargs)
        def a(self, value):
            """."""
            self.l.append(value)

    obj = A()
    obj.a()  # pylint: disable=E1120
    return obj.l
